      real function ran(ix)
      implicit integer*4 (a-z)
      real*8 x,s
*
*     A random number generator using the recursion IX=IX*A MOD P
*     where A=7**5 and P=2**31-1. The value returned is in the 
*     range 0.<= ran <1.
*
*     In this form fairly portable as does not require knowledge
*     of data storage, but does not adhere to the standard in two respects:
*     1) Assumes an integer word length of at least 32 bits 
*     2) Assumes that a positive integer less than 2**16 may be
*        floated without loss of digits.
*
*     This code is based on code published by Linus Schrage in
*     T.O.M.S. vol.5 no.2 June 1979 (pp 132-138)
*
*     The method employed is a multiplicative congruential one using a 
*     multiplier of 7**5 and taking the modulo to 2**31-1, i.e. the
*     generator number, x, is updated on each call to the value
*     x*7**5  modulo (2**31-1). The result returned is calculated as a 
*     real number having the value x/(2**31-1)  
*
      parameter (a=16807)
      parameter (b15=32768)
      parameter (b16=65536)
      parameter (p=2147483647)
      parameter (s=1d0/p)
*     a=7**5, b15=2**15, b16=2**16, p= 2**31-1
*
*     Get the 15 high order bits and 16 low order bits of ix
*
      xhi=ix/b16
      xlo=ix-xhi*b16
*
*     Multiply low order part
*
      xalo=xlo*a
*
*     Get high order part of product to carry
*
      leftlo=xalo/b16
*
*     And so obtain high order part of product
*
      xahi=xhi*a+leftlo
*
*     Obtain 32nd bit (overflow) of full product
*
      k=xahi/b15
*
*     Put all the bits of the product together and subtract p
*     (Must be in this form to prevent overflow. The ()'s are essential)
*
      ix=(((xalo-leftlo*b16 )-p) +(xahi-k*b15)*b16) +k
*
*     If <0 add p back again
*
      if (ix.lt.0) ix=ix+p
*
*     Finally multiply by 1/(2**31-1) to obtain number in range 0-1
*
      xhi=ix/b16
      x=dble(xhi)*65536.0d0+dble(ix-xhi*b16)
      x=x*s
      ran=sngl(x)
      return
      end
